@@ -1,6 +1,6 @@
 ;;; ellama-blueprint.el --- Working with blueprints -*- lexical-binding: t; package-lint-main-file: "ellama.el"; -*-
 
-;; Copyright (C) 2023-2025  Free Software Foundation, Inc.
+;; Copyright (C) 2023-2026  Free Software Foundation, Inc.
 
 ;; Author: Sergey Kostyaev <sskostyaev@gmail.com>
 ;; SPDX-License-Identifier: GPL-3.0-or-later
@@ -37,6 +37,70 @@
   :group 'ellama
   :type '(repeat plist))
 
+(defcustom ellama-blueprint-global-dir
+  (expand-file-name "ellama/blueprints" user-emacs-directory)
+  "Global directory for storing blueprint files."
+  :group 'ellama
+  :type 'directory)
+
+(defcustom ellama-blueprint-local-dir "blueprints"
+  "Local directory name for project-specific blueprints.
+When set to a string like \"blueprints\", it will look for this
+directory in the current project root."
+  :group 'ellama
+  :type 'string)
+
+(defcustom ellama-blueprint-file-extensions '("ellama-blueprint" "blueprint")
+  "File extensions recognized as blueprint files."
+  :group 'ellama
+  :type '(repeat string))
+
+(defun ellama-blueprint-get-local-dir ()
+  "Get local blueprint directory for current project."
+  (let ((project-root (ellama-tools-project-root-tool)))
+    (expand-file-name ellama-blueprint-local-dir project-root)))
+
+(defun ellama-blueprint-find-files (&optional dir extensions)
+  "Find blueprint files in DIR with given EXTENSIONS."
+  (let* ((search-dir (or dir ellama-blueprint-global-dir))
+         (exts (or extensions ellama-blueprint-file-extensions))
+         (files '()))
+    (when (file-exists-p search-dir)
+      (dolist (ext exts)
+        (setq files (append files
+			    (directory-files-recursively
+			     search-dir (concat "\\." ext "\\'"))))))
+    files))
+
+(defun ellama-blueprint-load-from-files ()
+  "Load blueprints from files."
+  (let ((global-files (ellama-blueprint-find-files ellama-blueprint-global-dir))
+        (local-files (ellama-blueprint-find-files (ellama-blueprint-get-local-dir)))
+        blueprints)
+    (dolist (file (append global-files local-files))
+      (when-let ((content (ellama-blueprint-read-file file))
+                 (name (file-name-sans-extension (file-name-nondirectory file)))
+                 (prompt (string-trim content)))
+        (push `(:act ,name :prompt ,prompt :file ,file) blueprints)))
+    blueprints))
+
+(defun ellama-blueprint-read-file (file)
+  "Read blueprint content from FILE."
+  (when (file-exists-p file)
+    (with-temp-buffer
+      (insert-file-contents file)
+      (buffer-string))))
+
+(defun ellama-blueprint-get-all-sources ()
+  "Get blueprints from all sources."
+  (let ((file-blueprints (ellama-blueprint-load-from-files))
+        (variable-blueprints ellama-blueprints)
+        (community-blueprints (ellama-community-prompts-ensure)))
+    (seq-uniq
+     (append file-blueprints variable-blueprints community-blueprints)
+     (lambda (b1 b2)
+       (string= (plist-get b1 :act) (plist-get b2 :act))))))
+
 ;;;###autoload
 (defun ellama-blueprint-set-system-kill-buffer ()
   "Set system message from current buffer and kill it."
@@ -62,10 +126,6 @@
   "{\\([[:alnum:]_-]+\\)}"
   "Regular expression to match blueprint variables like {var_name}.")
 
-(defvar ellama-blueprint-font-lock-keywords
-  '((,ellama-blueprint-variable-regexp 1 'font-lock-keyword-face))
-  "Highlight variables in curly braces for Ellama Blueprint Mode.")
-
 ;;;###autoload
 (define-derived-mode ellama-blueprint-mode
   text-mode
@@ -73,7 +133,7 @@
   "Toggle Ellama Blueprint mode."
   :keymap ellama-blueprint-mode-map
   :group 'ellama
-  (setq font-lock-defaults '(((,ellama-blueprint-variable-regexp 1 font-lock-keyword-face t))))
+  (setq font-lock-defaults `(((,ellama-blueprint-variable-regexp 1 font-lock-keyword-face t))))
   (setq header-line-format
 	(concat
 	 (propertize
@@ -102,6 +162,9 @@
 		    (define-key m [mode-line mouse-1] #'ellama-kill-current-buffer)
 		    m)))))
 
+;;;###autoload
+(add-to-list 'auto-mode-alist '("\\.ellama-blueprint\\'" . ellama-blueprint-mode))
+
 (defvar ellama-blueprint-buffer "*ellama-blueprint-buffer*"
   "Buffer for prompt blueprint.")
 
@@ -151,13 +214,8 @@ ARGS contains keys for fine control.
 	 (collection (pcase source
 		       ('user ellama-blueprints)
 		       ('community (ellama-community-prompts-ensure))
-		       (_ (seq-union
-			   ellama-blueprints
-			   (ellama-community-prompts-ensure)
-			   (lambda (blueprint1 blueprint2)
-			     (string=
-			      (plist-get blueprint1 :act)
-			      (plist-get blueprint2 :act)))))))
+		       ('files (ellama-blueprint-load-from-files))
+		       (_ (ellama-blueprint-get-all-sources))))
 	 selected-act
 	 selected-prompt)
     ;; Collect unique acts from the filtered collection
@@ -211,6 +269,7 @@ corresponding prompt is inserted into a blueprint buffer."
 	 (for-devs (y-or-n-p "For developers? "))
 	 (content (buffer-substring-no-properties (point-min) (point-max)))
 	 (blueprint `(:act ,name :prompt ,content :for-devs ,for-devs)))
+    (ellama-blueprint-remove name)
     (add-to-list 'ellama-blueprints blueprint t)
     (customize-save-variable 'ellama-blueprints ellama-blueprints)))
 
@@ -252,5 +311,20 @@ corresponding prompt is inserted into a blueprint buffer."
       (let ((value (read-string (format "Enter value for {%s}: " var))))
 	(ellama-blueprint-set-variable var value)))))
 
+;;;###autoload
+(defun ellama-blueprint-remove (act)
+  "Remove user defined blueprint with ACT name.
+ACT should be the act string of the blueprint to remove.
+If called interactively, prompts the user to select a blueprint."
+  (interactive
+   (let ((acts (mapcar (lambda (bp) (plist-get bp :act)) ellama-blueprints)))
+     (list (completing-read "Remove blueprint: " acts nil t))))
+  (let ((found (cl-remove-if (lambda (bp) (string= act (plist-get bp :act))) ellama-blueprints)))
+    (if (eq (length found) (length ellama-blueprints))
+        (message "No blueprint named '%s' found" act)
+      (setq ellama-blueprints found)
+      (customize-save-variable 'ellama-blueprints ellama-blueprints)
+      (message "Removed blueprint '%s'" act))))
+
 (provide 'ellama-blueprint)
 ;;; ellama-blueprint.el ends here.
